// BaseNabidka.cpp: implementation of the CBaseNabidka class.
//
//////////////////////////////////////////////////////////////////////

#include "BaseNabidka.h"
#include "TridaKteraRidiHru.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBaseNabidka::CBaseNabidka()
{
	znicit=budeKonec=nadpis[0]=vybranaNabidka=vybranaNabidkaOdkud=0;
	typObj= TYPOBJ_NABIDKA;
}

CBaseNabidka::~CBaseNabidka() {}

int CBaseNabidka::InitObject()
{
	vybranaNabidka= 0;
	for (int i=nabPoc-1; i>=0; i--) if ( nab[i].typ!=T_SKRYTA ) vybranaNabidka= i;
	posunRamecku= 0;

	return 1;
}

void CBaseNabidka::UpdateObjLogic(Uint8 *keys)
{
	if (keys[SDLK_UP]) {
		keys[SDLK_UP]=0;
		PosunNahoru();
	}
	if (keys[SDLK_DOWN]) {
		keys[SDLK_DOWN]=0;
		PosunDolu();
	}

	posunRamecku*= 0.955f;
}

void CBaseNabidka::PosunDolu()
{
	int vybPom= vybranaNabidka;
	do vybranaNabidka= ( vybranaNabidka+1 ) % nabPoc;
	while ( nab[vybranaNabidka].typ==T_SKRYTA );
	posunRamecku+= (vybranaNabidka-vybPom)*(VEL_ZNAKUY+TLO_MEZ)*ZV;
}
void CBaseNabidka::PosunNahoru()
{
	int vybPom= vybranaNabidka;
	do vybranaNabidka= ( vybranaNabidka-1+nabPoc ) % nabPoc;
	while ( nab[vybranaNabidka].typ==T_SKRYTA );
	posunRamecku+= (vybranaNabidka-vybPom)*(VEL_ZNAKUY+TLO_MEZ)*ZV;
}

void CBaseNabidka::UpdateObjDisplay()
{
	float r,g,b;
	for (int i=0; i<nabPoc; i++) {
		r=g=b=1;
		switch ( nab[i].typ ) {
		case T_NORMALNI: r=g=b=1; break;
		case T_SKRYTA: r=g=b=.30f; break;
		case T_SESIPKAMA: break;
		case T_KOUPIT: r=g=b=.70f; break;
		case T_PRODAT: r=g=b=1; break;
		case T_MODRA: r=g=.5; b=1; break;
		}
		if ( vybranaNabidka==i ) { r=1; g=b=.5f; }
		pogl->glPrintEx(5.5-4*TLO_RAM-VEL_ZNAKUX*ZV*strlen(nab[i].str), ZV*((nabPoc*VEL_ZNAKUY+(nabPoc-1)*TLO_MEZ)/2-VEL_ZNAKUY-(VEL_ZNAKUY+TLO_MEZ)*i),
			r,g,b,1.5, "%s", nab[i].str);
	}

	// ramecek kolem cely nabidky
	UdelejRamecek(5.5f-4*TLO_RAM-VEL_ZNAKUX*ZV*velNejdNab, -ZV*(nabPoc*VEL_ZNAKUY+(nabPoc-1)*TLO_MEZ)/2,
	VEL_ZNAKUX*ZV*velNejdNab, ZV*(nabPoc*VEL_ZNAKUY+(nabPoc-1)*TLO_MEZ));
	// ramecek kolem vybrany polozky
	UdelejRamecek(5.5f-4*TLO_RAM-VEL_ZNAKUX*ZV*velNejdNab, ZV*((nabPoc*VEL_ZNAKUY+(nabPoc-1)*TLO_MEZ)/2-VEL_ZNAKUY-(VEL_ZNAKUY+TLO_MEZ)*vybranaNabidka)+posunRamecku,
		VEL_ZNAKUX*ZV*velNejdNab, VEL_ZNAKUY*ZV);

	// nadpis
	pogl->glPrintEx( -(double)strlen(nadpis)/2*VEL_ZNAKUX*ZVN, TXTSCRY-1.5*VEL_ZNAKUY*ZVN, 1,1,1,ZVN, "%s", nadpis);
}




void CBaseNabidka::UdelejCtverecSTexturou(float x, float y, float w, float h)
{
	glBegin(GL_QUADS);												// Start Drawing A Textured Quad
		glTexCoord2f(0, 0); glVertex3f( x  , y  , -10);	// Bottom Left
		glTexCoord2f(1, 0); glVertex3f( x+w, y  , -10);	// Bottom Right
		glTexCoord2f(1, 1); glVertex3f( x+w, y+h, -10);	// Top Right
		glTexCoord2f(0, 1); glVertex3f( x  , y+h, -10);	// Top Left
	glEnd();														// Done Drawing The Quad
}

void CBaseNabidka::UdelejRamecek(float x, float y, float w, float h)
{
	// zadaji se souradnice textu, ktery se ma ohranicit, a okolo nej se udela ramecek
	// textury s rohama sou o polovinu vetsi nez s rovnejma hranama
	glBlendFunc(GL_ONE, GL_ONE);

	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR6]);
	UdelejCtverecSTexturou(x-TLO_RAM, y-TLO_RAM, 1.5f*TLO_RAM, 1.5f*TLO_RAM);		//  levo dole
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR7]);
	UdelejCtverecSTexturou(x+0.5f*TLO_RAM, y-TLO_RAM, w-TLO_RAM, 1.5f*TLO_RAM);			// dole
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR8]);
	UdelejCtverecSTexturou(x+w-0.5f*TLO_RAM, y-TLO_RAM, 1.5f*TLO_RAM, 1.5f*TLO_RAM);// pravo dole
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR4]);
	UdelejCtverecSTexturou(x-TLO_RAM, y+0.5f*TLO_RAM, 1.5f*TLO_RAM, h-TLO_RAM);			// levo
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR5]);
	UdelejCtverecSTexturou(x+w-0.5f*TLO_RAM, y+0.5f*TLO_RAM, 1.5f*TLO_RAM, h-TLO_RAM);				// pravo
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR1]);
	UdelejCtverecSTexturou(x-TLO_RAM, y+h-0.5f*TLO_RAM, 1.5f*TLO_RAM, 1.5f*TLO_RAM);// levo nahore
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR2]);
	UdelejCtverecSTexturou(x+0.5f*TLO_RAM, y+h-0.5f*TLO_RAM, w-TLO_RAM, 1.5f*TLO_RAM);				// nahore
	glBindTexture(GL_TEXTURE_2D, ptkrh->textury[TEX_MR3]);
	UdelejCtverecSTexturou(x+w-0.5f*TLO_RAM, y+h-0.5f*TLO_RAM, 1.5f*TLO_RAM, 1.5f*TLO_RAM);	// pravo nahore
}

void CBaseNabidka::UdelejCtverec(float x, float y, float w, float h)
{
	glBegin(GL_QUADS);												// Start Drawing A Textured Quad
		glVertex3f( x  , y  , -10);	// Bottom Left
		glVertex3f( x+w, y  , -10);	// Bottom Right
		glVertex3f( x+w, y+h, -10);	// Top Right
		glVertex3f( x  , y+h, -10);	// Top Left
	glEnd();														// Done Drawing The Quad
}
