// OGLfunc.cpp: implementation of the COGLfunc class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdarg.h>
#include <SDL_image.h>
#include "OGLfunc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

double myRand() { return (double)(rand()%10000)/10000; }


COGLfunc::COGLfunc() {}

COGLfunc::~COGLfunc()
{
	glDeleteLists(baseFont,128);	// Delete All 256 Display Lists
}

GLvoid COGLfunc::ReSizeGLScene(GLsizei width, GLsizei height)
{									// Resize And Initialize The GL Window
	if (height==0) height=1;		// Prevent A Divide By Zero By
	glViewport(0, 0, width, height);// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);	// Select The Projection Matrix
	glLoadIdentity();				// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
	//glOrtho(0.0f,width,height,0.0f,-100.0f,100.0f);
	//glOrtho(-20,20,-20,20,0,100);

	glMatrixMode(GL_MODELVIEW);		// Select The Modelview Matrix
	glLoadIdentity();				// Reset The Modelview Matrix
}

int COGLfunc::InitGL()				// All Setup For OpenGL Goes Here
{
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Black Background
	glClearDepth(1.0f);				// Depth Buffer Setup
	glDepthFunc(GL_LEQUAL);			// The Type Of Depth Testing To Do
	glEnable(GL_DEPTH_TEST);		// Enables Depth Testing
	glShadeModel(GL_SMOOTH);		// Enable Smooth Shading
	glEnable(GL_COLOR_MATERIAL);	// Enable Material Coloring
	glEnable(GL_TEXTURE_2D);		// Enable Texture Mapping ( NEW )
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);		// Really Nice Point Smoothing

	//lights
	/*static GLfloat LightAmbient[]= { 0.5f, 0.5f, 0.5f, 1.0f }; //vsude
	static GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f }; //ze zdroje
	static GLfloat LightPosition[]= { 0.0f, 0.0f, 1.0f, 1.0f };				 // Light Position 
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);		// Setup The Ambient Light
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		// Setup The Diffuse Light
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	// Position The Light/**/
	glEnable(GL_LIGHT0);

	//blending
	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);	// Full Brightness.  50% Alpha
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE);	// Set The Blending Function For Translucency
	glBlendFunc(GL_ONE, GL_ONE);
	glEnable(GL_BLEND);					// Enable Blending

	BuildFont();						// Build The Font

	return 1;					// Initialization Went OK
}

////////////////////Textury/////////////////////
SDL_Surface* COGLfunc::LoadBitmap(const char *filename)
{
	Uint8 *rowhi, *rowlo;			// Ukazatele na prohazovni dk
	Uint8 *tmpbuf, tmpch;			// Doasn pam
	int i, j;						// dc promnn pro cykly
	SDL_Surface *image;				// Natan obrzek
	image = IMG_Load(filename);		// Naten dat obrzku
	if (image == NULL) {			// Oeten chyby pi natn
		fprintf(stderr, "Nepodarilo se nacist %s: %s\n", filename, SDL_GetError());
		return(NULL);
	}

	tmpbuf = (Uint8 *)malloc(image->pitch);		// Alokace pamti
	if (tmpbuf == NULL) {						// Oeten chyby
		fprintf(stderr, "Nedostatek pameti\n");
		return NULL;
	}

	// Nastaven prvnho a poslednho dku
	rowhi = (Uint8 *)image->pixels;
	rowlo = rowhi + (image->h * image->pitch) - image->pitch;
	for (i = 0; i < image->h/2; i++) {
		// Pevrcen BGR na RGB
		if (image->format->Bshift == 0) {
			for (j = 0; j < image->w; j++) {
				tmpch = rowhi[j*3];
				rowhi[j*3] = rowhi[j*3+2];
				rowhi[j*3+2] = tmpch;
				tmpch = rowlo[j*3];
				rowlo[j*3] = rowlo[j*3+2];
				rowlo[j*3+2] = tmpch;
			}
		}
		// Prohozen dk
		memcpy(tmpbuf, rowhi, image->pitch);
		memcpy(rowhi, rowlo, image->pitch);
		memcpy(rowlo, tmpbuf, image->pitch);
		// Posun ukazatel na dky
		rowhi += image->pitch;
		rowlo -= image->pitch;
	}

	free(tmpbuf);							// klid
	return image;							// Vrt naten obrzek
}

int COGLfunc::LoadGLTextures(GLuint texture, char *Filename, int min_filter, int mag_filter, int mipmaps)
{
	SDL_Surface *surface;					// Obrzek
	surface = LoadBitmap(Filename);			// Naten obrzku
	if (surface == NULL)					// Oeten chyby
		return 0;

	glBindTexture(GL_TEXTURE_2D, texture);	// Nastaven textury

	// Nastaven poadovanch filtr
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter);

	if (mipmaps)							// Mipmapovan textura
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
	else									// Obyejn textura
		glTexImage2D(GL_TEXTURE_2D, 0, 3, surface->w, surface->h, 0, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);

	SDL_FreeSurface(surface);				// Smazn SDL_Surface
	surface = NULL;							// Nastaven ukazatele na NULL
	return 1;
}

/*////////////////////Textury2/////////////////////
AUX_RGBImageRec *COGLfunc::LoadBMP(char *Filename)
{
	FILE *File=NULL;

	if (!Filename) {
		fprintf(stderr, "Neni zadano meno textury!\n");
		return NULL;
	}
	
	File=fopen(Filename,"r");		// Check To See If The File Exists
	if (!File) { 
		fprintf(stderr, "Soubor %s s texturou neexistuje\n", Filename);
		return NULL; 
	}
	fclose(File);

	return auxDIBImageLoad(Filename);	// Load The Bitmap And Return A Pointer
}

int COGLfunc::LoadGLTextures(GLuint texture,char *Filename)
{
	int Status=FALSE;							// Status Indicator
	AUX_RGBImageRec *TextureImage[1];			// Create Storage Space For The Texture
	
	memset(TextureImage,0,sizeof(void *)*1);

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if ( TextureImage[0]=LoadBMP(Filename)) {
		Status=TRUE;

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, texture);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	}

	if (TextureImage[0]) {
		if (TextureImage[0]->data) free(TextureImage[0]->data);
		free(TextureImage[0]);
	}
	return Status;
}*/

////////////////////Pismo/////////////////////
void COGLfunc::BuildFont()					// Build Our Bitmap Font
{
	glGenTextures(1, &texture);
	LoadGLTextures(texture,"Data/Font.jpg");

	baseFont=glGenLists(128);							// Creating 256 Display Lists
	for (int loop1=0; loop1<128; loop1++)					// Loop Through All 256 Lists
	{
		float cx=float(loop1%16)/16.0f;					// X Position Of Current Character
		float cy=float(loop1/16)/16.0f;					// Y Position Of Current Character

		glNewList(baseFont+loop1,GL_COMPILE);				// Start Building A List
			glBegin(GL_QUADS);					// Use A Quad For Each Character
				glTexCoord2f(cx,        1.0f-cy-0.0625f);	glVertex2d(0  ,0  );// Bottom Left
				glTexCoord2f(cx+0.0625f,1.0f-cy-0.0625f);	glVertex2d(0.2,0  );// Bottom Right
				glTexCoord2f(cx+0.0625f,1.0f-cy);			glVertex2d(0.2,0.2);// Top Right
				glTexCoord2f(cx,        1.0f-cy);			glVertex2d(0  ,0.2);// Top Left
			glEnd();						// Done Building Our Quad (Character)
			glTranslated(VEL_ZNAKUX,0,0);					// Move To The Right Of The Character
		glEndList();							// Done Building The Display List
	}									// Loop Until All 256 Are Built
}

GLvoid COGLfunc::glPrint(GLdouble x, GLdouble y, const char *fmt, ...)
{
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments

	if (fmt == NULL) return;		// If There's No Text, Do Nothing

	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text

	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, texture);				// Select Our Font Texture
	glBlendFunc(GL_ONE, GL_ONE);
	glTranslated(x,y,-10);			// Position The Text (0,0 - Bottom Left)
	glPushAttrib(GL_LIST_BIT);		// Pushes The Display List Bits		( NEW )

	glListBase(baseFont-32);			// Sets The Base Character to 32
	glCallLists(strlen(text),GL_UNSIGNED_BYTE, text);	// Write The Text To The Screen

	glPopAttrib();					// Pops The Display List Bits	( NEW )
	glPopMatrix();
}

GLvoid COGLfunc::glPrintEx(GLdouble x, GLdouble y, float r, float g, float b, float zvetseni, const char *fmt, ...)
{
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments

	if (fmt == NULL) return;		// If There's No Text, Do Nothing

	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text

	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, texture);				// Select Our Font Texture
	glBlendFunc(GL_ONE, GL_ONE);
	glColor3f(r,g,b);
	glTranslated(x,y,-10);			// Position The Text (0,0 - Bottom Left)
	glScalef( zvetseni, zvetseni, 1);
	glPushAttrib(GL_LIST_BIT);		// Pushes The Display List Bits		( NEW )

	glListBase(baseFont-32);			// Sets The Base Character to 32
	glCallLists(strlen(text),GL_UNSIGNED_BYTE, text);	// Write The Text To The Screen

	glPopAttrib();					// Pops The Display List Bits	( NEW )
	glColor3f(1,1,1);
	glPopMatrix();
}

/*////////////////////Pismo2/////////////////////
// je pro nej potreba promena baseFont, ale je to ???windozacky???, takze to delam jinak
//GLuint	baseFont;		// Base Display List For The Font Set

void COGLfunc::BuildFont()					// Build Our Bitmap Font
{
	HFONT	oldfont,font;						// Windows Font ID
	HDC hDC=GetDC(NULL);

	baseFont = glGenLists(96);					// Storage For 96 Characters ( NEW )

	font = CreateFont(	-20,				// Height Of Font ( NEW )
				0,				// Width Of Font
				0,				// Angle Of Escapement
				0,				// Orientation Angle
				FW_BOLD,			// Font Weight
				FALSE,				// Italic
				FALSE,				// Underline
				FALSE,				// Strikeout
				ANSI_CHARSET,			// Character Set Identifier
				OUT_TT_PRECIS,			// Output Precision
				CLIP_DEFAULT_PRECIS,		// Clipping Precision
				ANTIALIASED_QUALITY,		// Output Quality
				FF_DONTCARE|DEFAULT_PITCH,	// Family And Pitch
				"Courier New");			// Font Name

	oldfont = (HFONT)SelectObject(hDC, font);		// Selects The Font We Want
	wglUseFontBitmaps(hDC, 32, 96, baseFont);			// Builds 96 Characters Starting At Character 32
	SelectObject(hDC, oldfont);				// Selects The Font We Want
	DeleteObject(font);					// Delete The Font
}

GLvoid COGLfunc::glPrint3d(float x, float y, float z, float r, float g, float b, const char *fmt, ...)				// Custom GL "Print" Routine
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor3f(r,g,b);
	glRasterPos3f(x,y,z);
	
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments
	if (fmt == NULL)				// If There's No Text
		return;						// Do Nothing
	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text
	glPushAttrib(GL_LIST_BIT);		// Pushes The Display List Bits		( NEW )
	glListBase(baseFont - 32);		// Sets The Base Character to 32	( NEW )
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text	( NEW )
	glPopAttrib();					// Pops The Display List Bits	( NEW )

	glColor3f(1,1,1);
}

GLvoid COGLfunc::glPrint2d(float x, float y, float r, float g, float b, const char *fmt, ...)				// Custom GL "Print" Routine
{
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments
	if (fmt == NULL)				// If There's No Text
		return;						// Do Nothing
	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text

	glPrint3d(x*0.55f,y*0.41f,-1.001f,r,g,b,text);
}
*/
